#include <stdio.h>
#include <stdarg.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "progress_msg.h"

#include "pixmaps/info.xpm"
#include "pixmaps/ok.xpm"
#include "pixmaps/fail.xpm"
#include "pixmaps/warn.xpm"


#define MAX_MSG_SIZE  512


/* widgets and icons */
static GtkWidget *w_main = NULL;
static GtkCList *clist_table[3];
static GdkPixmap *pixmap_table[4];
static GdkBitmap *mask_table[4];


/* public functions */
void pm_init(GladeXML *xml)
{
	/* widgets */
	w_main = glade_xml_get_widget(xml, "w_main");
	clist_table[PM_TAG_PROGRESS] = GTK_CLIST(glade_xml_get_widget(xml, "clist_tag_progress"));
	clist_table[PM_RENAME_PROGRESS] = GTK_CLIST(glade_xml_get_widget(xml, "clist_rename_progress"));
	clist_table[PM_PLAYLIST_PROGRESS] = GTK_CLIST(glade_xml_get_widget(xml, "clist_playlist_progress"));

	gtk_clist_set_column_justification(clist_table[PM_TAG_PROGRESS], 0, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification(clist_table[PM_RENAME_PROGRESS], 0, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification(clist_table[PM_PLAYLIST_PROGRESS], 0, GTK_JUSTIFY_RIGHT);

	/* icons */
/*	pixmap_table[PM_ICON_INFO] = 
		gdk_pixmap_create_from_xpm(w_main->window, &mask_table[PM_ICON_INFO], 
		NULL, "pixmaps/info.xpm");
	pixmap_table[PM_ICON_OK] = 
		gdk_pixmap_create_from_xpm(w_main->window, &mask_table[PM_ICON_OK], 
		NULL, "pixmaps/ok.xpm");
	pixmap_table[PM_ICON_FAIL] = 
		gdk_pixmap_create_from_xpm(w_main->window, &mask_table[PM_ICON_FAIL], 
		NULL, "pixmaps/fail.xpm");
	pixmap_table[PM_ICON_WARN] = 
		gdk_pixmap_create_from_xpm(w_main->window, &mask_table[PM_ICON_WARN], 
		NULL, "pixmaps/warn.xpm");*/
	pixmap_table[PM_ICON_INFO] = 
		gdk_pixmap_create_from_xpm_d(w_main->window, &mask_table[PM_ICON_INFO], 
		NULL, info_xpm);
	pixmap_table[PM_ICON_OK] = 
		gdk_pixmap_create_from_xpm_d(w_main->window, &mask_table[PM_ICON_OK], 
		NULL, ok_xpm);
	pixmap_table[PM_ICON_FAIL] = 
		gdk_pixmap_create_from_xpm_d(w_main->window, &mask_table[PM_ICON_FAIL], 
		NULL, fail_xpm);
	pixmap_table[PM_ICON_WARN] = 
		gdk_pixmap_create_from_xpm_d(w_main->window, &mask_table[PM_ICON_WARN], 
		NULL, warn_xpm);
}

void pm_start(gint list)
{
	gtk_clist_clear(clist_table[list]);
}

void pm_end(gint list)
{
	gint w = gtk_clist_optimal_column_width(clist_table[list], 1);
	gtk_clist_set_column_width(clist_table[list], 1, w);
}

void pm_printf(gint list, gint icon, const gchar *format, ...)
{
	va_list ap;
	gchar *rows_text[] = {NULL, NULL};
	gchar text[MAX_MSG_SIZE];
	gint row;

	va_start(ap, format);
	vsnprintf(text, MAX_MSG_SIZE, format, ap);
	va_end(ap);
	text[MAX_MSG_SIZE-1] = 0;

	rows_text[1] = text;
	row = gtk_clist_append(clist_table[list], rows_text);
	gtk_clist_set_selectable(clist_table[list], row, FALSE);
	if (icon != PM_ICON_NONE)
		gtk_clist_set_pixmap(clist_table[list], row, 0, pixmap_table[icon], mask_table[icon]);
}

void pm_scroll_to_bottom(gint list)
{
	/*gtk_clist_moveto(clist_table[list], 1000000, 0, 0.0, 0.0);*/

	GtkAdjustment *adj;

	adj = gtk_clist_get_vadjustment(clist_table[list]);
	gtk_adjustment_set_value(adj, adj->upper - adj->page_size);

}
