#ifndef STR_UTIL_H
#define STR_UTIL_H


/* text encodings */
enum {
	LOCALE     = -1,	// whatever the locale default is
	UTF_8      = 0,		// UTF-8
	UTF_16     = 1,		// UTF-16 with BOM
	UTF_16BE   = 2,		// UTF-16 big-endian (without BOM)
	UTF_16LE   = 3,		// UTF-16 little-endian (without BOM)
	ISO_8859_1 = 4		// ISO-8859-1, aka LATIN1
};


/* types of case conversion */
enum {
	CASE_CONV_NONE     = 0,
	CASE_CONV_LOWER    = 1,
	CASE_CONV_UPPER    = 2,
	CASE_CONV_SENTENCE = 3,
	CASE_CONV_TITLE    = 4
};


/*
 * Convert a string from one encoding to another
 * Returns a newly allocated string or NULL on failure
 */
char *str_convert_encoding(int from, int to, const char *str);

/*
 * Converts a string to the requested case.  Conversion is done in place.
 */
void str_convert_case(char *str, int conv);

/* 
 * Trims trailing <c> characters from string <str>
 */
void str_trim(char *str, char c);

/*
 * Replaces all occurrences of the character <r> by the character <c>
 */
void str_replace_char(char *str, char r, char c);

/*
 * Replaces all occurrences of characters in <r> by the character <c>
 */
void str_replace_chars(char *str, const char *r, char c);

/*
 * Removes all occurrences of character <rem> from <str>
 */
void str_remove_char(char *str, char rem);

/*
 * Returns a pointer to the <n>th occurrence of <c> in <s>, counting from 
 * the beginning.
 * Similar to strchr(3)
 */
char *str_strnchr(const char *s, char c, int n);

/*
 * Returns a pointer to the <n>th occurrence of <c> in <s>, counting from 
 * the end.
 * Similar to strrchr(3)
 */
char *str_strnrchr(const char *s, char c, int n);

/*
 * Safe version of strncpy, string is always NULL terminated.
 */
char *str_safe_strncpy(char *dest, const char *src, size_t n);


#endif
